"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadAppModule = void 0;
const tslib_1 = require("tslib");
const schematics_1 = require("@angular-devkit/schematics");
const schematics_2 = require("@angular/cdk/schematics");
const config_1 = require("@angular/cli/utilities/config");
const lazy_get_decorator_1 = require("lazy-get-decorator");
const ts = require("typescript");
const findNgModuleMetadata_1 = require("./findNgModuleMetadata");
const lazyGetter = (0, lazy_get_decorator_1.LazyGetter)();
class LoadedAppModuleImpl {
    constructor(_tree, _projectName) {
        this._tree = _tree;
        this._projectName = _projectName;
    }
    get appModule$() {
        return this.modulePath$
            .then(p => {
            const moduleFileContent = this._tree.read(p);
            if (!moduleFileContent) {
                throw new schematics_1.SchematicsException(`Could not read app module ${p}`);
            }
            return ts.createSourceFile(p, moduleFileContent.toString('utf8'), ts.ScriptTarget.Latest, true);
        });
    }
    get modulePath$() {
        return this.project$
            .then(p => (0, schematics_2.getAppModulePath)(this._tree, (0, schematics_2.getProjectMainFile)(p)));
    }
    get ngModuleMetadata$() {
        return this.appModule$
            .then(appM => {
            const value = (0, findNgModuleMetadata_1.findNgModuleMetadata)(appM);
            if (!value) {
                return this.modulePath$
                    .then(p => {
                    throw new schematics_1.SchematicsException(`Could not find NgModule declaration inside ${p}`);
                });
            }
            return value;
        });
    }
    get project$() {
        return this.workspace$
            .then(ws => (0, schematics_2.getProjectFromWorkspace)(ws, this._projectName));
    }
    get workspace$() {
        return (0, config_1.getWorkspace)('local');
    }
}
(0, tslib_1.__decorate)([
    lazyGetter
], LoadedAppModuleImpl.prototype, "appModule$", null);
(0, tslib_1.__decorate)([
    lazyGetter
], LoadedAppModuleImpl.prototype, "modulePath$", null);
(0, tslib_1.__decorate)([
    lazyGetter
], LoadedAppModuleImpl.prototype, "ngModuleMetadata$", null);
(0, tslib_1.__decorate)([
    lazyGetter
], LoadedAppModuleImpl.prototype, "project$", null);
(0, tslib_1.__decorate)([
    lazyGetter
], LoadedAppModuleImpl.prototype, "workspace$", null);
function loadAppModule(tree, projectName) {
    return new LoadedAppModuleImpl(tree, projectName);
}
exports.loadAppModule = loadAppModule;
//# sourceMappingURL=loadAppModule.js.map